%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    TESTING APPROXIMATE SOLUTIONS TO THE CME        %%%%%%%%%%%%
%%%%%%%%%%%          KS TESTING OF SCHLOGL MODEL               %%%%%%%%%%%%
%%%%%%%%%%%          Authors: Jenkinson/Goutsias               %%%%%%%%%%%%
%%%%%%%%%%%           Last Modified: 02/24/2013                %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

rng(123); % seed the random number generator for repeatable results

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

x0     = 600;
gamma2 = 1;
gamma3 = 2;
V      = 80;
k1     = 3;
k2     = 0.6;
k3     = 2.95;
k4     = 0.25;

tf = 1; % final simulation time

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve the macroscopic equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

y0 = x0/V;

[mu_time,mu] = ode23s(@(t,mu)k1*gamma2*(mu^2)-k2*(mu^3)-k3*mu...
                        +k4*gamma3,[0;tf],y0);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve the Lyapunov equation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[c_time,c]=ode23s(@(t,c)rhsLNA(t,c,mu_time,mu,gamma2,gamma3,k1,k2,...
                    k3,k4),mu_time,0);

mu_LNA = mu(end)*V;
c_LNA  = c(end)*V;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use adaptive algorithm to design the hypothesis test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

r_0 = 0.9;
r_1 = 0.99;
b_0 = 0.95;
b_1 = 0.05;

mu2star  = 17.1295; % mean of second Gaussian
var2star = 29.1812; % variance of second Gaussian

tic;
[L,alpha] = findLalpha(mu_LNA,sqrt(c_LNA),r_0,r_1,b_0,b_1,mu2star,...
                       sqrt(var2star));
toc;
                             
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Draw Monte Carlo samples and perform KS test
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tic;
XX = monteCarlo(gamma2,gamma3,V,k1,k2,k3,k4,x0,tf,L);
toc;

[H,pValue,KSstatistic,cv] = kstest((XX-mu_LNA)/sqrt(c_LNA),[],alpha); 

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve master equation numerically
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

timeStep = 0.01;

[Q,probs] = Schlogl(tf,timeStep,gamma2,gamma3,V,k1,k2,k3,k4,x0);

F_exact = cumsum(probs);

%
% Compute Kullback-Liebler Distance
%

X = 0:1000;
LNAprobs = normpdf(X,mu_LNA,sqrt(c_LNA));

KLdis = 0;
for x=0:1000
    if probs(x+1)>0
        KLdis = KLdis + log2(probs(x+1)/LNAprobs(x+1))*probs(x+1);
    end
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot decision band 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                             

s0 = sqrt(-0.5*log(alpha/2))/(sqrt(L)+0.12+0.11/sqrt(L));

[sampleCDF,x] = ecdf(XX);

X = 0:700;
LNAcdf = normcdf(X,mu_LNA,sqrt(c_LNA));
figure;
plot(X,LNAcdf,'LineWidth',1.25)
hold on
plot(X,min(LNAcdf+s0,1),'r','LineWidth',1.25)
plot(X,max(LNAcdf-s0,0),'r','LineWidth',1.25)
stairs(x,sampleCDF,'c','Linewidth',1.25)
hold off
xlabel('x')
ylabel('Pr[ X_1(1) \leq x ]')
title('{\bf DECISION BAND}')
axis([min(XX) max(XX) 0 1.05])
saveas(gcf,'results/decisionBand.fig')

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot confidence band
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

X = min(XX):max(XX);
LNAcdf = normcdf(X,mu_LNA,sqrt(c_LNA));

figure;
stairs(x,min(sampleCDF+s0,1),'g','Linewidth',1.25)
hold on;
stairs(x,max(sampleCDF-s0,0),'g','Linewidth',1.25)
stairs(X,F_exact(X+1),'k','LineWidth',1.25)
plot(X,LNAcdf,'Linewidth',1.25)
hold off;
axis([min(XX) max(XX) 0 1.05])
xlabel('x');ylabel('Pr[ X_1(1) \leq x ]')
title('{\bf CONFIDENCE BAND}')
saveas(gcf,'results/confidenceBand.fig')

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot true pdf vs. LNA pdf
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

X = 0:1000;
LNAprobs = normpdf(X,mu_LNA,sqrt(c_LNA));

figure
plot(X,LNAprobs,'LineWidth',1.25)
hold on
stairs(X,probs(X+1),'k','LineWidth',1.25)
title('{\bf TRUE (black) vs. LNA (blue) pdf}')
saveas(gcf,'results/PDFs.fig')